@@ -1,5 +1,6 @@
 from abc import ABC, abstractmethod
 from dataclasses import dataclass
+from enum import Enum
 from pathlib import Path
 from typing import Generic, List, Optional, Tuple, Type, TypeVar
 
@@ -11,7 +12,7 @@ from mohou.encoder import ImageEncoder
 from mohou.encoding_rule import EncodingRule, EncodingRuleBase
 from mohou.model import LSTM, PBLSTM
 from mohou.model.common import ModelT
-from mohou.model.lstm import LSTMBaseT
+from mohou.model.experimental import DisentangleLSTM, ProportionalModel
 from mohou.trainer import TrainCache
 from mohou.types import ElementDict, RGBImage, TerminateFlag
 from mohou.utils import detect_device
@@ -37,15 +38,33 @@ PropagatorT = TypeVar("PropagatorT", bound="PropagatorBase")
 
 
 class PropagatorBase(ABC, Generic[ModelT]):
+    propagator_model: ModelT
+    encoding_rule: EncodingRuleBase
+
     @classmethod
     @abstractmethod
     def create_default(cls: Type[PropagatorT], porject_path: Path) -> PropagatorT:
         pass
 
+    @property
+    @abstractmethod
+    def require_static_context(self) -> bool:
+        pass
 
-class LSTMPropagatorBase(PropagatorBase[LSTMBaseT]):
-    propagator_model: LSTMBaseT
-    encoding_rule: EncodingRuleBase
+    @abstractmethod
+    def feed(self, elem_dict: ElementDict) -> None:
+        pass
+
+    @abstractmethod
+    def predict(self, n_prop: int) -> List[ElementDict]:
+        pass
+
+    @abstractmethod
+    def reset(self) -> None:
+        pass
+
+
+class LSTMPropagatorBase(PropagatorBase[ModelT]):
     fed_state_list: List[np.ndarray]  # eatch state is equipped with flag
     n_init_duplicate: int
     is_initialized: bool
@@ -56,7 +75,7 @@ class LSTMPropagatorBase(PropagatorBase[LSTMBaseT]):
 
     def __init__(
         self,
-        lstm: LSTMBaseT,
+        lstm: ModelT,
         encoding_rule: EncodingRuleBase,
         n_init_duplicate: int = 0,
         prop_hidden: bool = False,
@@ -77,9 +96,7 @@ class LSTMPropagatorBase(PropagatorBase[LSTMBaseT]):
         self.is_initialized = False
         self.predicted_terminated_flags = []
 
-        require_static_context = lstm.config.n_static_context > 0
-
-        if not require_static_context:  # auto set
+        if not self.require_static_context:  # auto set
             self.static_context = np.empty((0,))
 
     def is_terminated(self) -> bool:
@@ -93,14 +110,18 @@ class LSTMPropagatorBase(PropagatorBase[LSTMBaseT]):
 
     @property
     def require_static_context(self) -> bool:
-        return self.propagator_model.config.n_static_context > 0
+        if hasattr(self.propagator_model.config, "n_static_context"):
+            return self.propagator_model.config.n_static_context > 0
+        else:
+            return False
 
     def set_static_context(self, value: np.ndarray) -> None:
+        assert hasattr(self.propagator_model.config, "n_static_context")
         assert value.ndim == 1
         assert self.propagator_model.config.n_static_context == len(value)
         self.static_context = value
 
-    def feed(self, elem_dict: ElementDict):
+    def feed(self, elem_dict: ElementDict) -> None:
         self._feed(elem_dict)
         if not self.is_initialized:
             for _ in range(self.n_init_duplicate):
@@ -109,7 +130,7 @@ class LSTMPropagatorBase(PropagatorBase[LSTMBaseT]):
         if not self.is_initialized:
             self.is_initialized = True
 
-    def _feed(self, elem_dict: ElementDict):
+    def _feed(self, elem_dict: ElementDict) -> None:
         if TerminateFlag not in elem_dict:
             elem_dict[TerminateFlag] = TerminateFlag.from_bool(False)
         state_with_flag = self.encoding_rule.apply(elem_dict)
@@ -169,7 +190,7 @@ class LSTMPropagatorBase(PropagatorBase[LSTMBaseT]):
         self.propagator_model.put_on_device(device)
 
     @abstractmethod
-    def _forward(self, state: np.ndarray) -> Tuple[torch.Tensor, torch.Tensor]:
+    def _forward(self, states: np.ndarray) -> Tuple[torch.Tensor, torch.Tensor]:
         pass
 
 
@@ -242,3 +263,67 @@ class PBLSTMPropagator(LSTMPropagatorBase[PBLSTM]):
             states_torch, pb_torch, context_torch, self._hidden
         )
         return out, hidden
+
+
+class DisentangleLSTMPropagator(LSTMPropagatorBase[DisentangleLSTM]):
+    @classmethod
+    def create_default(cls, project_path: Path) -> "DisentangleLSTMPropagator":
+        tcache = TrainCache.load(project_path, DisentangleLSTM)
+        encoding_rule = EncodingRule.create_default(project_path)
+        return cls(tcache.best_model, encoding_rule)
+
+    def _forward(self, states: np.ndarray) -> Tuple[torch.Tensor, torch.Tensor]:
+        def numpy_to_unsqueezed_torch(arr: np.ndarray) -> torch.Tensor:
+            device = self.get_device()
+            return torch.from_numpy(arr).float().unsqueeze(dim=0).to(device)
+
+        states_torch = numpy_to_unsqueezed_torch(states)
+        pred_torch = self.propagator_model.forward(states_torch)
+        hidden = torch.empty((0,)).to(self.get_device())  # dummy
+        return pred_torch, hidden
+
+
+@dataclass
+class ProportionalModelPropagator(PropagatorBase[ProportionalModel]):
+    model: ProportionalModel
+    encoding_rule: EncodingRuleBase
+    fed_vector: Optional[np.ndarray] = None
+
+    @classmethod
+    def create_default(cls, project_path: Path) -> "ProportionalModelPropagator":
+        tcache = TrainCache.load(project_path, ProportionalModel)
+        encoding_rule = EncodingRule.create_default(project_path)
+        return cls(tcache.best_model, encoding_rule)
+
+    @property
+    def require_static_context(self) -> bool:
+        # currently not support static context
+        return False
+
+    def feed(self, elem_dict: ElementDict) -> None:
+        feature_vector = self.encoding_rule.apply(elem_dict)
+        self.fed_vector = feature_vector
+
+    def predict(self, n_prop: int) -> List[ElementDict]:
+        assert self.fed_vector is not None
+        feeding_vector_list = [self.fed_vector]
+        for _ in range(n_prop):
+            latest_vector = feeding_vector_list[-1]
+            torch_vector = torch.from_numpy(latest_vector).float()
+            seq_sample = torch_vector.unsqueeze(0).unsqueeze(0)
+            pred_sample = self.model.forward(seq_sample).squeeze()
+            feeding_vector_list.append(pred_sample.cpu().detach().numpy())
+        pred_vector_list = feeding_vector_list[1:]
+        pred_edict_list = [self.encoding_rule.inverse_apply(vec) for vec in pred_vector_list]
+        return pred_edict_list
+
+    def reset(self) -> None:
+        pass
+
+
+class PropagatorSelection(Enum):
+    lstm = LSTMPropagator
+    pblstm = PBLSTMPropagator
+    chimera = ChimeraPropagator
+    proportional = ProportionalModelPropagator
+    disentangle = DisentangleLSTMPropagator
